#include "smgbalib.h"
#include "graphics.h"
#include "game.h"
#include "gauges.h"

// private - game board
cell cells[6][6];
// private - mapping from FOOD_BITMAP_ to the actual bitmaps
u16* foodBitmaps[12];
// private - game status
u8 status;
// private - this drives the difficulty (how quickly you get hungry)
u16 amountEaten;
// private - cursor position
signed char cursorX, cursorY;

// private - setup initial array
void InitializeCells( void );
// private - display cursor
void DisplayCursor( void );
// private - A was pressed
void HandleFoodSelection( void );
// private - check for game over, or adjust score and hunger
void VerifyThresholds( void );
// private - show game over screen
void GameOver( void );

void PlayGame( void ) {

	InitializeCells();
	
	SetPalette( (u16*)mainBDPalette );
	status = STATUS_PLAYING;
	amountEaten = 0;
	// current video frame counter (increments every time the screen refreshes)
	// it also drives the hunger meter
	u16 currentFrame = 0;
	
	// counters
	u16 i,j;
	
	// temp variable, used for hunger speed
	u16 levelDivider;

	// main loop - should never exit it
	while(1) {
		currentFrame++;

		// draw background!
		DrawFullScreenOpaque( (u16*)backgroundData );
		
		// cursor
		DisplayCursor();
		
		// draw all 6x6 foods
		for( i = 0; i < GAME_BOARD_SIDE; i++ ) {
			for( j = 0; j < GAME_BOARD_SIDE; j++ ) {
				DrawBitmapTransparent( GAME_BOARD_OFFSET_X + 12 * i, GAME_BOARD_OFFSET_Y + 24 * j,
							   food_HEIGHT, food_WIDTH,
							   cells[i][j].foodBitmap );
			}
		}

		// adjust hunger
		levelDivider = 4 - amountEaten / 10;
		if( currentFrame % levelDivider == 0 ) {
			GaugeHungerLevel--;
		}
		
		DisplayScore();
		DisplayFoodTypeGauge();
		DisplayHungerGauge();
		
		Flip();

		// DONE DISPLAYING!
		
		if( status != STATUS_PLAYING )break;
		
		if( IS_A_PRESSED ) {
			HandleFoodSelection();
		}
		VerifyThresholds();
		
		if( IS_LEFT_PRESSED ) cursorX--;
		if( IS_RIGHT_PRESSED ) cursorX++;
		if( IS_UP_PRESSED ) cursorY--;
		if( IS_DOWN_PRESSED ) cursorY++;
		
		if( cursorX > 4 ) cursorX = 0;
		if( cursorX < 0 ) cursorX = 4;
		
		if( cursorY > 4 ) cursorY = 0;
		if( cursorY < 0 ) cursorY = 4;
		
	}

	// handle game over
	
	// keep game board visible for a bit longer
	PlayNote( NOTE_G2, 128 );Sleep( 128 );
	PlayNote( NOTE_FSHARP2_GFLAT2, 128 );Sleep( 128 );
	PlayNote( NOTE_F2, 128 );Sleep( 128 );
	PlayNote( NOTE_E2, 255 );Sleep( 512 );
	
	if( Score > HighScore )
		HighScore = Score;
	
	GameOver();
}

void DisplayCursor( void ) {

	DrawBitmapOpaque( GAME_BOARD_OFFSET_X + 12 * cursorX, GAME_BOARD_OFFSET_Y + 24 * cursorY,
					  48, 48,
					  (u16*)cursorData );
}

void InitializeCells( void ) {
	// first assign all food bitmaps
	foodBitmaps[ FOOD_BITMAP_BURGER ] = (u16*)burgerData;
	foodBitmaps[ FOOD_BITMAP_DONUT ] = (u16*)donutData;
	foodBitmaps[ FOOD_BITMAP_FRIES ] = (u16*)friesData;
	foodBitmaps[ FOOD_BITMAP_HOTDOG ] = (u16*)hotdogData;
	foodBitmaps[ FOOD_BITMAP_PIZZA ] = (u16*)pizzaData;
	foodBitmaps[ FOOD_BITMAP_TACO ] = (u16*)tacoData;
	
	foodBitmaps[ FOOD_BITMAP_CARROT ] = (u16*)carrotData;
	foodBitmaps[ FOOD_BITMAP_GREENPEPPER ] = (u16*)greenbellpepperData;
	foodBitmaps[ FOOD_BITMAP_LEMON ] = (u16*)lemonData;
	foodBitmaps[ FOOD_BITMAP_MUSHROOM ] = (u16*)mushroomData;
	foodBitmaps[ FOOD_BITMAP_GRAPES ] = (u16*)purplegrapeData;
	foodBitmaps[ FOOD_BITMAP_TURNIP ] = (u16*)turnipData;
	
	cursorX = 2;
	cursorY = 2;
	
	GaugeFoodTypeLevel = 50;
	GaugeHungerLevel = 20;
	Score = 0;
	
	u16 x, y;
	s32 random;
	
	// fill our game board with food
	for( x = 0 ; x < 6 ; x++ ) {
		for( y = 0 ; y < 6 ; y++ ) {
			// we have 12 possible foods
			random = NextRandom( 12 );
			cells[x][y].foodType = (u8)random;
			cells[x][y].foodBitmap = foodBitmaps[(u8)random];
		}
	}
}

void GenerateNewFood( u8 x, u8 y ) {
	s32 random;
	random = NextRandom( 12 );
	cells[x][y].foodType = (u8)random;
	cells[x][y].foodBitmap = foodBitmaps[(u8)random];
}

void HandleFoodSelection( void ) {

	u16 totalUnhealthy = 0;
	u16 totalBland = 0;
	
	// cap it at 40
	if( amountEaten < 40 )
		amountEaten++;
	
	if( cells[cursorX][cursorY].foodType < 6 )
		totalUnhealthy++;
	else
		totalBland++;
		
	if( cells[cursorX+1][cursorY].foodType < 6 )
		totalUnhealthy++;
	else
		totalBland++;
		
	if( cells[cursorX][cursorY+1].foodType < 6 )
		totalUnhealthy++;
	else
		totalBland++;
		
	if( cells[cursorX+1][cursorY+1].foodType < 6 )
		totalUnhealthy++;
	else
		totalBland++;
	
	// four of a kind
	if( totalUnhealthy == 4 )
		GaugeFoodTypeLevel -= GAUGE_FOOD_TYPE_4_OF_A_KIND;
	
	if( totalBland == 4 )
		GaugeFoodTypeLevel += GAUGE_FOOD_TYPE_4_OF_A_KIND;
	
	// three of a kind
	if( totalUnhealthy == 3 )
		GaugeFoodTypeLevel -= GAUGE_FOOD_TYPE_3_OF_A_KIND;
		
	if( totalBland == 3 )
		GaugeFoodTypeLevel += GAUGE_FOOD_TYPE_3_OF_A_KIND;
	
	// balanced
	int modifier = GAUGE_FOOD_TYPE_2_OF_A_KIND;
	if( NextRandom( 2 ) == 0 )
		modifier = -1 * modifier;
	if( totalUnhealthy == 2 )
		GaugeFoodTypeLevel += modifier;
		
	VerifyThresholds();
	
	if( status == STATUS_PLAYING ) {
		// adjust score - the faster you eat, the faster you get points
		u16 scoreModifier = GaugeHungerLevel / 4;
		
		// prevent huge scores via quick tapping on the same spot
		// by putting less emphasis on speed (highest hunger levels count the same as medium ones)
		if( scoreModifier > 3 ) scoreModifier=3;
		
		
		// also, if you eat 4 or 3 of a kind, you get better scores
		if( totalUnhealthy == 4 || totalBland == 4 )
			scoreModifier = scoreModifier * 4;
		else if( totalUnhealthy == 3 || totalBland == 3 )
			scoreModifier = scoreModifier * 2;
		else 
			scoreModifier = scoreModifier / 2;
		
		
		
		Score += scoreModifier;
		
		// reset hunger
		GaugeHungerLevel = 20;
		
		// generate new foods to replace eaten ones
		GenerateNewFood( cursorX, cursorY );
		GenerateNewFood( cursorX+1, cursorY );
		GenerateNewFood( cursorX, cursorY+1 );
		GenerateNewFood( cursorX+1, cursorY+1 );
		
		PlayNote( NOTE_E3, 32 );Sleep(32);
		PlayNote( NOTE_E3, 32 );Sleep(32);
		PlayNote( NOTE_B3, 32 );Sleep(32);
		
		PlayNote( NOTE_E3, 32 );Sleep(32);
		PlayNote( NOTE_E3, 32 );Sleep(32);
		PlayNote( NOTE_ASHARP3_BFLAT3, 32 );Sleep(64);
	}
}

void VerifyThresholds( void ) {
	if( GaugeFoodTypeLevel < 1 ) {
		GaugeFoodTypeLevel = 0;
		status = STATUS_GAMEOVER_UNHEALTHY;
		return;
	}
	
	if( GaugeFoodTypeLevel > 100 ) {
		GaugeFoodTypeLevel = 101;
		status = STATUS_GAMEOVER_BLAND;
		return;
	}
	
	if( GaugeHungerLevel < 0 ) {
		GaugeHungerLevel = 0;
		status = STATUS_GAMEOVER_STARVED;
		return;
	}
}

void GameOver( void ) {
	if( status == STATUS_GAMEOVER_STARVED ) 
		ShowGameOverStarvedScreen();
	else if( status == STATUS_GAMEOVER_UNHEALTHY ) 
		ShowGameOverUnhealthyScreen();
	else if( status == STATUS_GAMEOVER_BLAND ) 
		ShowGameOverBlandScreen();
}
